﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Telerik.Pivot.Core;
using Telerik.Pivot.Core.Aggregates;
using Telerik.Pivot.Core.Fields;
using Telerik.WinControls.UI;

namespace _1051483_RadPivotGridDateTimeStep
{
    public partial class Form1 : Form
    {
        List<Order> orders = new List<Order>();
        LocalDataSourceProvider provider;

        public Form1()
        {
            InitializeComponent();
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            this.LoadData();
            this.SetupPivot();
        }

        private void SetupPivot()
        {
            this.provider = new LocalDataSourceProvider() { ItemsSource = this.orders };
            LocalDataSourceFieldDescriptionsProvider descriptionProvider = new LocalDataSourceFieldDescriptionsProvider();
            this.provider.PrepareDescriptionForField += provider_PrepareDescriptionForField;
            this.provider.FieldDescriptionsProvider = descriptionProvider;

            provider.RowGroupDescriptions.Add(new PropertyGroupDescription()
            {
                PropertyName = "Product"
            });

            provider.RowGroupDescriptions.Add(new HourQuarterGroupDescription()
            {
                PropertyName = "Date",
            });

            provider.ColumnGroupDescriptions.Add(new PropertyGroupDescription()
            {
                PropertyName = "Advertisement",
            });

            provider.AggregateDescriptions.Add(new PropertyAggregateDescription()
            {
                PropertyName = "Net",
                AggregateFunction = AggregateFunctions.Sum
            });

            provider.AggregatesPosition = PivotAxis.Rows;
            provider.AggregatesLevel = 21;

            this.radPivotGrid1.PivotGridElement.DataProvider = provider;
            this.radPivotGrid1.PivotGridElement.BestFitHelper.BestFitColumns();
            this.radPivotGrid1.PivotGridElement.BestFitHelper.BestFitRowHeaders();
        }

        private void provider_PrepareDescriptionForField(object sender, PrepareDescriptionForFieldEventArgs e)
        {
            if (e.Description.DisplayName == "Date" && !(e.Description is DateTimeGroupDescription))
            {
                e.Description = new HourQuarterGroupDescription()
                {
                    PropertyName = e.Description.DisplayName
                };
            }
        }

        private void LoadData()
        {
            Stream stream = System.Reflection.Assembly.GetAssembly(this.GetType()).GetManifestResourceStream("_1051483_RadPivotGridDateTimeStep.PivotData.txt");

            using (StreamReader streamReader = new StreamReader(stream))
            {
                Random r = new Random();
                while (streamReader.Peek() != -1)
                {
                    string[] items = streamReader.ReadLine().Split('\t');
                    Order o = new Order()
                    {
                        Date = DateTime.Parse(items[0], System.Globalization.CultureInfo.InvariantCulture).AddHours(r.Next(23)).AddMinutes(r.Next(1, 60)),
                        Product = items[1],
                        Quantity = int.Parse(items[2], System.Globalization.CultureInfo.InvariantCulture),
                        Net = double.Parse(items[3], System.Globalization.CultureInfo.InvariantCulture),
                        Promotion = items[4],
                        Advertisement = items[5],
                    };

                    if (items.Count() == 7)
                    {
                        o.Time = TimeSpan.Parse(items[6]);
                    }

                    orders.Add(o);
                }
            }
        }
    }
}
