﻿using System;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace _1051483_RadPivotGridDateTimeStep
{
    public struct HourQuarterGroup : IComparable, IComparable<HourQuarterGroup>, IEquatable<HourQuarterGroup>
    {
        private readonly CultureInfo culture;

        public HourQuarterGroup(int quarter)
            : this()
        {
            this.Quarter = quarter;
        }

        public HourQuarterGroup(int quarter, CultureInfo culture)
            : this(quarter)
        {
            this.culture = culture;
        }

        public int Quarter
        {
            get;
            set;
        }

        public override string ToString()
        {
            CultureInfo cultureToUse = this.culture ?? CultureInfo.InvariantCulture;

            string quarterAsString = this.Quarter.ToString(cultureToUse.NumberFormat);
            switch (quarterAsString)
            {
                case "1":
                    return "0-15";
                case "2":
                    return "15-30";
                case "3":
                    return "30-45";
                case "4":
                    return "45-60";
            }

            return string.Empty;
        }

        public override int GetHashCode()
        {
            return this.Quarter;
        }

        [SuppressMessage("Microsoft.Usage", "CA2231:OverloadOperatorEqualsOnOverridingValueTypeEquals", Justification = "Design choice.")]
        public override bool Equals(object obj)
        {
            if (obj is HourQuarterGroup)
            {
                return this.Equals((HourQuarterGroup)obj);
            }

            return false;
        }

        public bool Equals(HourQuarterGroup other)
        {
            return this.Quarter == other.Quarter;
        }

        public int CompareTo(object obj)
        {
            if (obj is HourQuarterGroup)
            {
                return this.CompareTo((HourQuarterGroup)obj);
            }

            throw new ArgumentException("Can not compare.", "obj");
        }

        public int CompareTo(HourQuarterGroup other)
        {
            return this.Quarter.CompareTo(other.Quarter);
        }

        public static bool operator <(HourQuarterGroup left, HourQuarterGroup right)
        {
            return left.CompareTo(right) < 0;
        }

        public static bool operator >(HourQuarterGroup left, HourQuarterGroup right)
        {
            return left.CompareTo(right) > 0;
        }

        public static bool operator <=(HourQuarterGroup left, HourQuarterGroup right)
        {
            return left.CompareTo(right) <= 0;
        }

        public static bool operator >=(HourQuarterGroup left, HourQuarterGroup right)
        {
            return left.CompareTo(right) >= 0;
        }

        public static bool operator ==(HourQuarterGroup left, HourQuarterGroup right)
        {
            return left.Equals(right);
        }

        public static bool operator !=(HourQuarterGroup left, HourQuarterGroup right)
        {
            return !left.Equals(right);
        }
    }
}
